/* overflow.js

	Purpose:
		
	Description:
		
	History:
		Aug 9, 2013 Created by Pao Wang

Copyright (C) 2013 Potix Corporation. All Rights Reserved.
*/

(function () {
	// constant
	var NUM_CELL = 0;
	var STR_CELL = 1;
	var FORMULA_CELL = 2;
	var BLANK_CELL = 3;
	var BOOLEAN_CELL = 4;
	var ERROR_CELL = 5;
	
	var oCell = {};
	
	zk.override(zss.Cell.prototype, oCell, {
		/**
		 * if textWidth <= cellWidth, no overflow
         * if textWidth > cellWidth &&  textWidth < availableSpace, overflow with cut text
         * if textWidth > availableSpace, overflow without cut
		 */
		_processOverflow: function() {
            var OVERFLOW_RIGHT_ALIGN = "zscell-overflow-right-alignment-text";
            var OVERFLOW_PARTIAL_RIGHT_ALIGN = "zscell-overflow-partial-right-alignment-text";
			//ZSS-944: 90 degree text will not do overflow
			var rotate90 = this.rotate == 90 || this.rotate == -90; //ZSS-1020
			if (rotate90) return;

			var col = this.c;
			var rBorder = this.rborder;
			var $n = jq(this.$n());

			var cave = this.$n('cave');
			var $cave = jq(cave);
			var tn = this.getTextNode();
			var $tn = jq(tn);

			// clear previous width setting for measuring correctly
			this.setCaveWidth('');
            //clear existing overflow CSS because it might change
            $cave.removeClass(OVERFLOW_RIGHT_ALIGN);
            $cave.removeClass(OVERFLOW_PARTIAL_RIGHT_ALIGN);
            $n.removeClass("zscell-overflow-b");
            $n.removeClass("zscell-overflow");

			var sheet = this.sheet;
			var custColWidth = sheet.custColWidth;
			var notxtwd = (this._txtwd == undefined || this._txtwd < 0);  //ZSS-1171
			var sw = notxtwd ? tn.scrollWidth : this._txtwd; // heavy duty (it's actually a function call from ZK widget), ZSS-1171
			var cellPad = sheet.cellPad;

			//ZSS-1171
			if (notxtwd) {
				this._txtwd = sw;
				sheet._sendOnTextWidth(this.r, col, sw);
			}

			// skip to process overflow if current cell's width is enough
			// right grid line won't be removed if it's css has no zscell-overflow class
			if(sw <= custColWidth.getSize(this.c) - cellPad) {
				return;
			}


			// apply overflow CSS and adjust cell width
			if (this.halign == 'l'){ //left alignment
                var availableSpace = this.getAvailableSpace(sw, function(cell){return cell.nextSibling;}) - cellPad;
				this.setCaveWidth(jq.px0(availableSpace));

				$n.addClass(rBorder ? "zscell-overflow-b" : "zscell-overflow");
			}else { //right alignment; ZSS-1338
				//count available space for overflow to the left-hand side cells
				var textWidth = zk.ie9_ ? sw : $cave.width();
				var availableSpace = this.getAvailableSpace(textWidth, function(cell){return cell.previousSibling;});
				var cellWidth = this.sheet.custColWidth.getSize(this.c);
				//make overflow
				if (availableSpace > textWidth){ //overflow without cut
				    //right align already moves text left, only decide overflow here
					$cave.addClass(OVERFLOW_RIGHT_ALIGN);
					$n.addClass("zscell-overflow-b");
				}else if (availableSpace > cellWidth){ //partially overflow
					$n.addClass("zscell-overflow-b");
					$cave.addClass(OVERFLOW_PARTIAL_RIGHT_ALIGN);
				    $cave.css('left', jq.px(cellWidth - availableSpace));
				    this.setCaveWidth(jq.px0(availableSpace));
				    $tn.css('left', jq.px(jq(this.$n()).width() - textWidth + (availableSpace - cellWidth)));
				}
			}
		},
		/**
		* shrink a text's width cut by an adjacent text.
		* set width on .zscelltext or .zscelltext-real according to browser version
		* @param width string in pixel or empty string
		*/
		setCaveWidth: function(widthPx){
            if (zk.ie6_ || zk.ie7_ || zk.ie9_) {
            // Note. For IE6/IE7 need to set cave's width (set text node width won't work)
                jq(this.$n('cave')).css('width', widthPx);
            } else {
                jq(this.getTextNode()).css('width', widthPx);
            }
		},
		/**
		* count available width for overflow by summing left/right adjacent cell width
		* getNextAdjacentCell: return a left/right adjacent cell upon right/left align
		*/
		getAvailableSpace: function(textWidth, getNextAdjacentCell){
		    var availableSpace = this.sheet.custColWidth.getSize(this.c); //cell width as initial space
            var cell = getNextAdjacentCell(this);
            while (cell && cell.cellType === BLANK_CELL && availableSpace < textWidth) {
                availableSpace += this.sheet.custColWidth.getSize(cell.c);
                cell = getNextAdjacentCell(cell);
            }
            return availableSpace;
		},
		_onProcessOverflow: function (evt) { //ZSS-1338
			if (this.overflow) {
				var row = this.r,
					data = evt.data;
				if (data) {
					var rCol = data.col,
						tRow = data.tRow,
						bRow = data.bRow;
					if ((tRow == undefined && bRow == undefined) || 
						(tRow && bRow && row >= tRow && row <= bRow)) {
						this._processOverflow(true);
					}
				}
			}
		},
		update_: function(updateSrc){ //ZSS-1338
			oCell.update_.apply(this, arguments);
			if (this.overflow){
			    //render overflow earlier than right alignment to compute the text width
			    this._processOverflow();
			}
			// only need to shift a text
            if (this.cellType == STR_CELL || this.cellType == FORMULA_CELL){
			    this.shiftAlignedText();
            }
		}
	});
	
})(); // end of topmost function
